@@ -74,7 +74,9 @@ Digikam::Digikam() : KMainWindow()
     slot_connect();
 
   setCamStatusMsg("Ready.");
-  
+
+  slot_photoFolderClicked( view->photoFolderTree->firstChild() );
+
 }
 
 Digikam::~Digikam()
@@ -289,7 +291,8 @@ void Digikam::readOptions()
 
    // camera bar icon text setting
   KToolBar::IconText camToolBarIconText;
-  camToolBarIconText = (KToolBar::IconText) config->readNumEntry("CamToolBarIconText", KToolBar::IconOnly);
+  camToolBarIconText = (KToolBar::IconText) config->readNumEntry("CamToolBarIconText", 
+								 KToolBar::IconOnly);
   toolBarCameraBrowser->setIconText(camToolBarIconText);
     
   QSize size=config->readSizeEntry("Geometry");
@@ -589,6 +592,7 @@ void Digikam::slot_showHideToolBars(QWidget* tab)
 
 void Digikam::slot_connect() {
   
+  camCancelAction->setEnabled(true);
   if (initCamera()!=0)  {
     setCamStatusMsg("Abort.");
     return;
@@ -623,6 +627,8 @@ void Digikam::slot_connect() {
     return;
   }
 
+  camCancelAction->setEnabled(false);
+
   setAbilitiesButtons();
 
   
@@ -635,7 +641,7 @@ void Digikam::slot_connect() {
     view->KListView_camFolders->setSelected(view->KListView_camFolders->firstChild(), true);
   }
 
-
+  setCamStatusMsg("Camera Initialized");
 
 }
 
@@ -719,7 +725,8 @@ void Digikam:: slot_getThumbNails(bool checked)
     }
   
   }
-  
+
+  setCamStatusMsg("Camera Initialized");  
 }
   
 
@@ -871,7 +878,7 @@ void Digikam:: slot_downloadImages()
   
   restoreCamButtons();
   setCamStatusMsg("Finished Downloads");
-  view->camProgressBar->setValue(0);
+  //  view->camProgressBar->setValue(0);
   view->camProgressBar->repaint();
 }
 
@@ -1010,7 +1017,7 @@ void Digikam::slot_downloadAllImages()
 
   restoreCamButtons();
   setCamStatusMsg("Finished Downloads");
-  view->camProgressBar->setValue(0);
+  //  view->camProgressBar->setValue(0);
   view->camProgressBar->repaint();
 
 }
@@ -1021,11 +1028,12 @@ void Digikam::slot_downloadAllImages()
 void Digikam::slot_stopDownloadImages()
 {
 
+  Gphoto_Status::my_cancel=true;
   stopDownloading = true;
   restoreCamButtons();
 
   setCamStatusMsg("Aborted");
-  view->camProgressBar->setValue(0);
+  //view->camProgressBar->setValue(0);
   view->camProgressBar->repaint();
 
 }
@@ -1236,6 +1244,20 @@ void Digikam::slot_setup()
 }
 
 
+void Digikam::slot_setCamStatusMsg(const QString &msg) {
+  //camStatusBar->message(msg,2000);
+  view->camStatusBar->changeItem(msg,0);
+  kapp->processEvents();
+}
+
+void Digikam::slot_setCamProgressBarValue(int value) {
+
+  view->camProgressBar->setValue(value);
+  kapp->processEvents();
+}
+
+
+
 void Digikam::slot_photoFolderClicked(QListViewItem *item) 
 {
   if (!item) return;
