@@ -60,7 +60,10 @@ class Digikam : public KMainWindow
   void slot_showHideToolBars(QWidget* name);
   void slot_startedGenThumbNails();
   void slot_stoppedGenThumbNails();
-   
+
+  void slot_setCamStatusMsg(const QString &msg);
+  void slot_setCamProgressBarValue(int value);
+
  private:
  
   void initView();
