@@ -20,10 +20,13 @@
 
 #include <qdir.h>
 
+#include <iostream.h>
+#include <strstream.h>
 
 Gphoto_Interface::Gphoto_Interface(bool setup){
 
   my_camera = NULL;
+  glob_context = NULL;
   my_cameraModel = "";
   my_cameraPort = "";
   my_cameraPortPath = "";
@@ -48,6 +51,10 @@ Gphoto_Interface::~Gphoto_Interface(){
   if (my_cameraInitialised) {
     gp_camera_unref(my_camera);
     my_camera = NULL;
+    if (glob_context) {
+      gp_context_unref(glob_context);
+      glob_context = NULL;
+    }
   }
 
 }
@@ -60,6 +67,8 @@ int Gphoto_Interface::setupCamera() {
   char model[64], port[128];
   int modelNum = -1, portNum = -1;
 
+  glob_context = gp_context_new ();
+
   gp_setting_get ("digikam", "model", model);
   gp_setting_get ("digikam", "port", port);
 
@@ -73,7 +82,7 @@ int Gphoto_Interface::setupCamera() {
 
   gp_camera_new(&my_camera);
   gp_abilities_list_new (&abilList);
-  gp_abilities_list_load (abilList);
+  gp_abilities_list_load (abilList, glob_context);
   gp_port_info_list_new (&infoList);
   gp_port_info_list_load (infoList);
 
@@ -140,6 +149,8 @@ int Gphoto_Interface::initCamera() {
   if (my_camera != NULL) {
     gp_camera_unref(my_camera);
     my_camera = NULL;
+    gp_context_unref(glob_context);
+    glob_context = NULL;
   }
 
   if (setupCamera() == -1) 
@@ -148,17 +159,16 @@ int Gphoto_Interface::initCamera() {
   qWarning(my_cameraModel);
   qWarning(my_cameraPort);
 
-  if (gp_camera_init(my_camera)<0) {
+  Gphoto_Status *gpStatus = new Gphoto_Status();
+
+  if (gp_camera_init(my_camera, gpStatus->my_context) != GP_OK) {
     gp_camera_unref(my_camera);
     my_camera = NULL;
+    delete gpStatus;
     return -2;
-  } 
-
-
-  gp_camera_set_status_func (my_camera, status_func, NULL);
-  gp_camera_set_progress_func (my_camera, progress_func, NULL);
-  gp_camera_set_message_func (my_camera, message_func, NULL);
+  }
 
+  delete gpStatus;
 
   my_cameraInitialised=true;
 
@@ -220,11 +230,16 @@ int Gphoto_Interface:: getSubFolders(QString folder, QStringList& subFolderList,
 
   gp_list_new(&folderList);
 
-  if (gp_camera_folder_list_folders (my_camera, folder, folderList)
+  Gphoto_Status *gpStatus = new Gphoto_Status();
+
+  if (gp_camera_folder_list_folders (my_camera, folder, folderList, gpStatus->my_context)
       != GP_OK) {
     gp_list_unref(folderList);
+
     return -1;
   }
+  delete gpStatus;
+
 
   numSubFolders = gp_list_count(folderList);
 
@@ -268,14 +283,18 @@ int Gphoto_Interface::getImageNames(QString folder, QStringList& imagesList,  QS
   imagesList.clear();
   imagesInfoList.clear();
 
+  Gphoto_Status *gpStatus = new Gphoto_Status();
 
   gp_list_new(&cameraListFiles);
-  if (gp_camera_folder_list_files(my_camera, folder, cameraListFiles)
+  if (gp_camera_folder_list_files(my_camera, folder, cameraListFiles, gpStatus->my_context)
       != GP_OK) {
     gp_list_unref(cameraListFiles);
+    delete gpStatus;
     return -1;
   }
 
+  delete gpStatus;
+
   int count = gp_list_count (cameraListFiles);
 
   for (int i = 0; i < count; i++) {
@@ -288,7 +307,7 @@ int Gphoto_Interface::getImageNames(QString folder, QStringList& imagesList,  QS
 
     CameraFileInfo info;
     if (gp_camera_file_get_info(my_camera, folder, 
-				cameraFileName, &info) == GP_OK) {
+				cameraFileName, &info, glob_context) == GP_OK) {
       if (info.file.fields & GP_FILE_INFO_TYPE)
 	imagesInfoList.append(QString(info.file.type));
       else
@@ -313,17 +332,22 @@ QPixmap* Gphoto_Interface::getThumbNail(QString folder, QString imageName)
   QPixmap* thumbNail;
   CameraFile *cameraFile;
   const char* data;
-  long int size;
+  unsigned long int size;
 
   gp_file_new(&cameraFile);
 
+  Gphoto_Status *gpStatus = new Gphoto_Status();
+
   if (gp_camera_file_get(my_camera, folder, imageName, GP_FILE_TYPE_PREVIEW, 
-			 cameraFile) != GP_OK) {
+			 cameraFile, gpStatus->my_context) != GP_OK) {
     qWarning("Failed to get preview");
     gp_file_unref(cameraFile);
+    delete gpStatus;
     return NULL;
   }
 
+  delete gpStatus;
+
 
   gp_file_get_data_and_size(cameraFile, &data, &size);
 
@@ -342,11 +366,16 @@ int Gphoto_Interface::downloadImage(QString folder, QString imageName, QString s
   CameraFile *cameraFile;
 
   gp_file_new(&cameraFile);
+
+  Gphoto_Status *gpStatus = new Gphoto_Status();
   if (gp_camera_file_get(my_camera, folder, imageName,
-			 GP_FILE_TYPE_NORMAL, cameraFile) != GP_OK) {
+			 GP_FILE_TYPE_NORMAL, cameraFile, gpStatus->my_context) != GP_OK) {
     gp_file_unref(cameraFile);
+    delete gpStatus;
     return -1;
   }
+  delete gpStatus;
+
   if (gp_file_save(cameraFile,saveFile) != GP_OK) {
     gp_file_unref(cameraFile);
     return -1;
@@ -360,7 +389,7 @@ int Gphoto_Interface::downloadImage(QString folder, QString imageName, QString s
 int Gphoto_Interface::deleteImage(QString folder, QString imageName)
 {
   
-  if (gp_camera_file_delete(my_camera, folder, imageName) != GP_OK)
+  if (gp_camera_file_delete(my_camera, folder, imageName, glob_context) != GP_OK)
     return -1;
   
   return 0;
@@ -372,7 +401,7 @@ int Gphoto_Interface::deleteImage(QString folder, QString imageName)
 int Gphoto_Interface :: makeDir(QString folder, QString newFolder)
 {
 
-  if (gp_camera_folder_make_dir(my_camera, folder, newFolder) != GP_OK)
+  if (gp_camera_folder_make_dir(my_camera, folder, newFolder, glob_context) != GP_OK)
     return -1;
 
   return 0;
@@ -423,28 +452,6 @@ bool Gphoto_Interface::cameraSupportsDelDir() {
 }
 
 
-void Gphoto_Interface::status_func (Camera *camera, const char *status, void *data)
-{
-
-  Gphoto_Messages::gpMessagesWrapper()->emit statusChanged(status);
-  
-}
-
-void Gphoto_Interface::message_func (Camera *camera, const char *message, void *data)
-{
-  qWarning(message);
-}
-
-void Gphoto_Interface::progress_func (Camera *camera, float percentage, void *data)
-{
-  if (percentage > 0.) 
-    
-    Gphoto_Messages::gpMessagesWrapper()->emit progressChanged(int(100*percentage));
-  
-
-}
-
-
 void Gphoto_Interface::debug_func (GPLogLevel level, const char *domain, const char *format,
 			  va_list args, void *data)
 {
@@ -467,7 +474,7 @@ void Gphoto_Interface::getSupportedCameras(QStringList& cameraList, int& numSupp
   CameraAbilities abil;
 
   gp_abilities_list_new (&abilList);
-  gp_abilities_list_load (abilList);
+  gp_abilities_list_load (abilList, glob_context);
   
   numSupported = gp_abilities_list_count (abilList);
   if (numSupported < 0) {
@@ -538,7 +545,7 @@ void Gphoto_Interface::getCameraPorts(QString cameraModel, QStringList& supporte
   CameraAbilitiesList *abilList;
   
   gp_abilities_list_new (&abilList);
-  gp_abilities_list_load (abilList);
+  gp_abilities_list_load (abilList, glob_context);
   i = gp_abilities_list_lookup_model (abilList, cameraModel.local8Bit().data());
   gp_abilities_list_get_abilities (abilList, i, &abilities);
   gp_abilities_list_free (abilList);
@@ -557,10 +564,10 @@ int Gphoto_Interface::autoDetectCamera(QString& cameraModel, QString& cameraPort
   const char *model, *port;
 
   gp_abilities_list_new (&abilList);
-  gp_abilities_list_load (abilList);
+  gp_abilities_list_load (abilList, glob_context);
   gp_port_info_list_new (&infoList);
   gp_port_info_list_load (infoList);
-  gp_abilities_list_detect (abilList, infoList, &camList);
+  gp_abilities_list_detect (abilList, infoList, &camList, glob_context);
   gp_abilities_list_free (abilList);
   gp_port_info_list_free (infoList);
 
@@ -589,7 +596,7 @@ void Gphoto_Interface::getCameraSummary(QString& cameraSummary)
 {
   CameraText summary;
   
-  gp_camera_get_summary(my_camera, &summary);
+  gp_camera_get_summary(my_camera, &summary, glob_context);
   cameraSummary = QString(summary.text);
 
 }
@@ -598,7 +605,7 @@ void Gphoto_Interface::getCameraManual(QString& cameraManual)
 {
   CameraText manual;
   
-  gp_camera_get_manual(my_camera, &manual);
+  gp_camera_get_manual(my_camera, &manual, glob_context);
   cameraManual = QString(manual.text);
 
 }
@@ -607,7 +614,99 @@ void Gphoto_Interface::getCameraAbout(QString& cameraAbout)
 {  
   CameraText about;
   
-  gp_camera_get_about(my_camera, &about);
+  gp_camera_get_about(my_camera, &about, glob_context);
   cameraAbout = QString(about.text);
 
 }
+
+
+// ------- Gphoto_Status -----------------------------------
+
+float Gphoto_Status::my_target=0.0;
+bool  Gphoto_Status::my_cancel=false;
+
+Gphoto_Status::Gphoto_Status () {
+
+  my_context = gp_context_new();
+  
+  my_cancel = false;
+
+  gp_context_set_cancel_func  (my_context, cancel_func, NULL);  
+  gp_context_set_error_func  (my_context, error_func, NULL);
+  gp_context_set_status_func (my_context, status_func, NULL); 
+  gp_context_set_progress_funcs (my_context, progress_start_func,
+				 progress_update_func, progress_stop_func, 
+				 NULL);
+}
+
+Gphoto_Status::~Gphoto_Status() {
+
+  if (my_context) {
+    gp_context_unref(my_context);
+    my_context = NULL;
+  }
+
+}
+
+GPContextFeedback Gphoto_Status::cancel_func (GPContext *context, void *data)
+{
+  return (my_cancel ? GP_CONTEXT_FEEDBACK_CANCEL : GP_CONTEXT_FEEDBACK_OK);
+}
+
+void Gphoto_Status::error_func(GPContext *context, const char *format, 
+			       va_list args, void *data) {
+
+  char buf[4096] = "";
+  int nSize = vsnprintf( buf, 4096, format, args );
+  if( nSize > 4094 ) nSize = 4094;
+  buf[nSize] = '\0';
+
+  QString qstr;
+  qstr = qstr.fromLocal8Bit(buf);
+  qWarning(qstr);
+
+}
+
+void Gphoto_Status::status_func (GPContext *context, const char *format, 
+				 va_list args, void *data) {
+  
+  char buf[4096] = "";
+  int nSize = vsnprintf( buf, 4096, format, args );
+  if( nSize > 4094 ) nSize = 4094;
+  buf[nSize] = '\0';
+
+  QString status;
+  status = status.fromLocal8Bit(buf);
+  qWarning("Cam Status: " + status);
+  Gphoto_Messages::gpMessagesWrapper()->emit statusChanged(status);
+
+}
+
+unsigned int Gphoto_Status::progress_start_func(GPContext *context, float target, const char *format,
+					 va_list args, void *data) {
+  char buf[4096] = "";
+  int nSize = vsnprintf( buf, 4096, format, args );
+  if( nSize > 4094 ) nSize = 4094;
+  buf[nSize] = '\0';
+
+  QString prog;
+  prog = prog.fromLocal8Bit(buf);
+  qWarning("Cam Progress: " + prog);
+
+  my_target = target;
+					 
+}
+
+void Gphoto_Status::progress_update_func(GPContext *context, unsigned int id, float current, 
+					  void *data) {
+
+  int percentage = int(100.0 * current/my_target);
+  Gphoto_Messages::gpMessagesWrapper()->emit progressChanged(percentage);					 
+}
+
+void  Gphoto_Status::progress_stop_func(GPContext *context, unsigned int id, void *data) {
+  
+  Gphoto_Messages::gpMessagesWrapper()->emit progressChanged(0);	 
+}
+
+
