@@ -32,6 +32,7 @@
 extern "C" {
 	#include <stdio.h>
 	#include <gphoto2-camera.h>
+	#include <gphoto2-context.h>
 	#include <gphoto2-abilities-list.h>
 	#include <gphoto2-file.h>	
 	#include <gphoto2-list.h>
@@ -82,6 +83,7 @@ class Gphoto_Interface {
 
     Camera *my_camera;
     CameraAbilities my_cameraAbilities;
+    GPContext *glob_context;
     QString my_cameraModel;
     QString my_cameraPort;
     QString my_cameraPortPath;
@@ -97,15 +99,38 @@ class Gphoto_Interface {
     int setupCamera();
     void setupCameraAbilities();
 
-    static void status_func (Camera *camera, const char *status, void *data);
-    static void message_func (Camera *camera, const char *message, void *data);
-    static void progress_func (Camera *camera, float percentage, void *data);
-
     static void debug_func (GPLogLevel level, const char *domain, const char *format,
 			    va_list args, void *data); 
 
 };
 
 
+class Gphoto_Status {
+
+ public:
+
+  Gphoto_Status();
+  ~Gphoto_Status();
+
+  GPContext *my_context;
+  static bool  my_cancel;
+
+  
+ private:
+
+  static GPContextFeedback cancel_func (GPContext *context, void *data);
+  static void error_func  (GPContext *context, const char *format, va_list args, void *data);
+  static void status_func (GPContext *context, const char *format, va_list args, void *data);
+  static unsigned int progress_start_func (GPContext *context, float target, const char *format,
+				   va_list args, void *data);
+  static void progress_update_func (GPContext *context, unsigned int id, float current, 
+				    void *data);
+  static void progress_stop_func(GPContext *context, unsigned int id, void *data);
+
+  static float my_target;
+
+    
+};
+
 
 #endif
