@@ -22,7 +22,7 @@ Gphoto_Messages* Gphoto_Messages::gpMessages=0;
 Gphoto_Messages* Gphoto_Messages::gpMessagesWrapper() {
   
   if (!gpMessages)
-    gpMessages = new Gphoto_Messages();
+    gpMessages = new Gphoto_Messages;
   return gpMessages;
 
 }
